.module/ram/abs=0      C1KNOB;
.pagelength  59;                    { Works for .LST files with 550C }
.include    <system.k>;

{******************************************************************************
 *
 *      EZ-KIT Lite DSP Program for SOSSD
 *	c1knob.dsp  -  Control functions for knob and display.
 *	RSL 15 June 1999
 *
 * This includes the subroutines for interacting with the Knob/Display
 * box, including the 4 switches and the 2 LED.  The main body of the program
 * reads the knob as fast as possible, about every 500 uSec.  The value of
 * a number is updated by the knob routine and is displayed about 12 times
 * per second.  The switches are read about 100 times per second and SW1
 * and SW2 control the red and green LEDs.  These are examples of using the
 * various routines.
 *
 *****************************************************************************}

{***********************   CONSTANTS     *************************************}
.const		TIME1 = 4000;
.const		TIME2 = 510;

{********************   PROGRAM MEMORY     ***********************************}
{ Place declarations of program memory here.  This is not for the program
  itself, but rather for tables of constants that can be accessed at the
  same time as variable data is being accsessed from data memory.     }

.var/pm	encoder[16];			{ Last state*4 + New state is adr and
					  output is the implied change }
.init	encoder: 0, H#FFFF00, H#000100, 0,
		 H#000100, 0, 0, H#FFFF00,
		 H#FFFF00, 0, 0, H#000100,
		 0, H#000100, H#FFFF00, 0;

{********************   DATA MEMORY     **************************************}
{ SPORT0, i.e., Serial PORT 0, is located in data memory.
  The AD1847 Codec hardware operates independently of the DSP.
  But when A/D conversion is complete there is an interrupt generated
  by SPORT0.  This occurs 48,000 times per second as determined
  by the crystal clock in the Codec.  This interrupt causes, among other
  things, a jump to location 14 Hex in program memory.  This, in turn,
  is sent off to process the newly received data, or to do any other task
  required on a periodic basis, such as generation of waveforms.
  It is at this time that new data can be sent to the Codec to be converted
  by the D/A in the Codec.  Memory locations tx_buf and rx_buf provide the
  interaction with the SPORT0 (and thus the Codec) when interrupts occur. }
.var/dm/circ	tx_buf[3];		{ SPORT0: Cmd + L data + R data }
.var/dm/circ 	rx_buf[3];		{ SPORT0: Status + L data + R data }

{ data16 is the 16 bits that will be clocked out to the hardware
  shift registers.  This is sent by the function load16.
  Many functions use data16.  Only modify the bits of interest.
	Bit 0		LSB of input address
	Bit 1		Middle bit of input address
	Bit 2		MSB of input address
	Bit 3		Green LED, 1=on
	Bit 4		Red LED, 1=on
	Bit 5		LCD Enable line
	Bit 6		LCD Read/Write Line, 0=Write
	Bit 7		LCD Data/Command line, 1=Data
	Bit 8		LCD Data, Bit 4
	BIt 9		LCD Data, Bit 5
	Bit 10		LCD Data, Bit 6
	BIt 11		LCD Data, Bit 7
	Bit 12 to 15	Spares, Not Used
}
.var/dm		data16;
.var/dm		sr_data;		{ Working register used by load16 }

{ Two software clocks are used to slow down the update rate of the LCD
  display and of the Switches.  This gives more time to update the knob
  data.  }
.var/dm		clock1, clock2;

.var/dm		knob_st;
.var/dm		sw3, sw4;
.var/dm		temp1;			{ These are available for any fcn }
.var/dm		temp2, temp3;
.var/dm		number;
.var/dm		digit[6];		{ ASCII number, sign + 5 BCD  }
.var/dm		msg[8];

{ Initialization of the AD1847 Codec is complex.  The internal Codec
  registers are transfered out of the following buffer.  It is circular to
  allow detection of a complete load of commands.     }
.var/dm/circ    init_cmds[13];		{ Codec Initialization }
.var/dm		stat_flag;		{ Codec Init status }

{********************  Interrupt vector table  *******************************}
	{ A jump to location 0 occurs when the dsp is reset }
        jump start;  rti; rti; rti;     {00: reset }

	{ Most of the interrupt vectors are not used, such as the next 3 }
        rti;         rti; rti; rti;     {04: IRQ2 }	{ Filler instr }
        rti;         rti; rti; rti;     {08: IRQL1 }
        rti;         rti; rti; rti;     {0c: IRQL0 }

	{ Do to the variable stat_flag, this interrupt routine is only used
	  during initialization of the AD1847 Codec.  It is used to send
	  register data to the Codec.  By using the interrupt the data is
	  not sent when the SPORT0 transmit buffer is full.         }
	ar = dm(stat_flag);		{10: SPORT0 tx }
        ar = pass ar;			{  see if needs initialization }
        if eq rti;			{ If dm(stat_flag)==0 exit this int }
        jump next_cmd;			{   else go transfer a control word }

	{ The following instructions are executed when an A/D conversion
	  cycle has been completed in the AD1847 Codec, and the resulting data
	  has been serially transmitted to the SPORT0.  The SPORT0 hardware
	  then generates an interrupt with an associated jump to this
	  program address: }
	jump input_samples;             {14: SPORT0 rx }
                     rti; rti; rti;	{Filler so that there are 4 instr }

	{ And more unused interrupt locations: }
        rti;         rti; rti; rti;     {18: IRQE }
        rti;         rti; rti; rti;     {1c: BDMA }
	rti;         rti; rti; rti;     {20: SPORT1 tx or IRQ1 }
        rti;         rti; rti; rti;     {24: SPORT1 rx or IRQ0 }
	rti;         rti; rti; rti;     {28: timer }
        rti;         rti; rti; rti;     {2c: power down }

{*****************************  START  ***************************************
 * Initialize process and drop into main loop.  None of the index registers
 * have been initialized, so we need to turn off all interrupts before
 * the hardware is allowed to generate one.  Then we are safe with instructions
 * like dm(i0, m0)=ar.  This is the dreaded uninitialized pointer problem!!
 ****************************************************************************}
start:	imask=0;			{ Turn off all interrupts }

	ax0 = 1234;
	dm(number) = ax0;

	ax0 = TIME1;
	dm(clock1) = ax0;
	ax0 = TIME2;
	dm(clock2) = ax0;

	{ The following loads msg[], an 8 character area that is loaded to the
	  right half of the LCD display by the function send_msg.       }
	ax0 = 84; dm(msg+0) = ax0;  { 'T' }
	ax0 = 69; dm(msg+1) = ax0;  { 'E' }
	ax0 = 83; dm(msg+2) = ax0;  { 'S' }
	ax0 = 84; dm(msg+3) = ax0;  { 'T' }
	ax0 = 32; dm(msg+4) = ax0;  { ' ' }
	ax0 = 77; dm(msg+5) = ax0;  { 'M' }
	ax0 = 83; dm(msg+6) = ax0;  { 'S' }
	ax0 = 71; dm(msg+7) = ax0;  { 'G' }

	{ Initialization has been broken into two pieces to simplify testing.
	  Most items should go into init0, but those that involve interrupts
	  can be put into init1 so that they can be commented out easily
	  when using the simulator.  }
	call init0;			{ Instructions that simulate easily }

	call init1;			{ And those that do not }

	{ Initialize LCD and start a load }
	ax0 = 0;			{ Start with all hardware shift }
	dm(data16) = ax0;		{   register outputs reset to 0  }
	call load16;

	ax0 = 1234;
	dm(number) = ax0;

	call init_lcd;

	call send_msg;

{****************     Main Program Loop    ************************************
 *  Main processing loop.  Does all non-periodic (background) processing.
 *  This loop is interrupted at a 48KHz rate to send and receive Codec data.
 *****************************************************************************}
again:
	call knob;			{ See if knob has moved (in ay0) }
	ar = dm(number);		{ So we can display the result }
	ar = ar+ay0;			{ Change number by -1, 0 or 1 }
	dm(number) = ar;

	{ clock1 and clock2 decrease by 1 every 20.8 usec in input_samples }
	{ Now, see if clock1 is to zero.  If so, display number. }
	ar = dm(clock1);
	ar = pass ar;
	if gt jump ag0;			{ Not time yet }
	ar = TIME1;			{ Reset clock1 to TIME1 }
	dm(clock1) = ar;
	ar=dm(number);
	call ar2lcd;			{ Display number in ar }

	{ Now, see if clock2 is to zero.  If so, update switches. }
ag0:	ar = dm(clock2);
	ar = pass ar;
	if gt jump ag4;
	ar = TIME2;
	dm(clock2) = ar;

	call in_sw1;
	if ne jump ag1;
	call red_on;
	jump ag1a;
ag1:	call red_off;
ag1a:	call in_sw2;
	if ne jump ag2;
	call grn_on;
	jump ag4;
ag2:	call grn_off;
ag4:	jump again;			{ Go round and round forever }

{***********************************************************************
 *                 SPORT0 Interrupt Handler                            *
 *  SPORT0 inputs from the Codec put us here 48000 times per second.   *
 *  Routines here are those that need this periodic service.           *
 ***********************************************************************}
input_samples:

	{ We do not know where calculations were at when the interrupt was
	  executed.  All of the processor registers that are used by the
	  background process must be protected, if they are to be used
	  by the interrupting routine, where we are now.  This is done by
	  having two sets of most registers and switching them with
	  the "ena sec_reg" instruction.  Note that the i index registers,
	  and the associated m and l registers are not switched.  There
	  is only one set of these 24 registers.   }
        ena sec_reg;                    { use secondary register bank }

	mr0=dm(rx_buf+1);		{ Get left audio from A/D (Codec) }
	mr1=dm(rx_buf+2);		{ Right }

	{ This shell does no processing to the signals, other than to pass
	  them through.  Processing would go here.   }

	dm(tx_buf+1) = mr0;		{ Send left audio to D/A (Codec) }
	dm(tx_buf+2) = mr1;		{ Right }

	ar = dm(clock1);		{ Slow down counter for LCD update }
	ar = ar - 1;
	if lt jump is8;
	dm(clock1) = ar;

is8:	ar = dm(clock2);		{ Slow down counter for Switch update }
	ar = ar - 1;
	if lt jump is9;
	dm(clock2) = ar;
is9:	nop;

	dis sec_reg;
	rti;				{ This undoes the interrupt }

{ LOAD16()  -  This routine can be called from the background to load the data
  in dm(data16) into the hardware shift regs.  Data from dm(data16)
  is unaltered by this routine.  Time to load is about 160 microseconds.  This
  could be much faster, but the delays allow for the rise-time of 
  feedthrough filters on the data, clock and latch wires.
}
load16:	ax0 = dm(data16);		
	dm(sr_data)=ax0;		{ 16 bits of shift data }
	call delay3;			{ Ensure rise of latch pulse }

	cntr = 16;
	do clk_bit until ce;
	    ax0=dm(sr_data);
	    ar=tstbit 15 of ax0;
	    sr0=ax0;
	    sr=lshift sr0 by 1 (lo);	{ Move bits over, status unaffected }
	    dm(sr_data)=sr0;		{ Save shifted data }
	    if eq jump bl4a;
	    ax0 = dm(PFDATA);		{ Data line to 1 }
	    ar = setbit 2 of ax0;
	    dm(PFDATA) = ar;
	    jump bl4e;
bl4a:	    ax0 = dm(PFDATA);		{ Data line to 0 }
	    ar = clrbit 2 of ax0;
	    dm(PFDATA) = ar;

bl4e:	    call delay3;
	    ax0 = dm(PFDATA);		{ Clock line }
	    ar = setbit 0 of ax0;
	    dm(PFDATA) = ar;
	    call delay3;
	    ax0 = dm(PFDATA);		{ Clock fall }
	    ar = clrbit 0 of ax0;
	    dm(PFDATA) = ar;
	    call delay3;
clk_bit:    nop;			{ Don't end loop on a call }

	ax0 = dm(PFDATA);		{ Latch the data }
	ar = setbit 1 of ax0;
	dm(PFDATA) = ar;
	call delay3;
	ax0 = dm(PFDATA);		{ Clear the latch for the next load }
	ar = clrbit 1 of ax0;
	dm(PFDATA) = ar;
	call delay3;
	rts;

{ LCD4()  -  Sends 4 bits to D4-D7 of LCD. These are the lower 4 bits of
  ar.  Bit 7 of ar is sent to RS of the LCD and is 1 for data and 0 for
  command.
}
lcd4:	sr = lshift ar by 8 (hi);
	af = tstbit 15 of sr1;		{ It was bit 7, now bit 15 }
	if ne jump lcd4a;
	ay1 = 0;			{ SR bit (0 = command) }
	jump lcd4b;
lcd4a:	ay1 = b#0000000010000000;	{ SR bit (1=data) }
lcd4b:	ay0 = b#0000111100000000;	{ Data bit mask }
	ar = sr1 and ay0;		{ Clear the SR bit from it's old spot }
	af = ar or ay1;			{ Add SR bit into new spot }
	ax0 = dm(data16);
	ay0 = b#1111000000011111;
	ar = ax0 and ay0;		{ Mask off lcd bits }
	ar = ar or af;
	dm(data16) = ar;
	call load16;			{ Send out 16 bits with enable low }
	ax0 = dm(data16);
	ar = setbit 5 of ax0;		{ Enable bit high to LCD }
	dm(data16) = ar;
	call load16;
	ax0 = dm(data16);
	ar = clrbit 5 of ax0;		{ Enable bit low to LCD }
	dm(data16) = ar;
	call load16;
CALL DELAY50;
	rts;

{ OUT_CMD()  -  Sends the 8 bit command in SR1 to the LCD.
  Note that RS is set to 0 for command by masking, if not already 0 }
out_cmd: dm(temp1) = sr1;		{ Save for second nibble }
	sr = lshift sr1 by -4 (hi);	{ Get high 4 bits }
	ay0 = b#00001111;		{ Mask to get onl these 4 bits }
	ar = sr1 and ay0;
	call lcd4;			{ Send 4 bits of command }
	ar = dm(temp1);			{ Get command again }
	ay0 = b#00001111;
	ar = ar and ay0;
	jump lcd4;

{ OUT_CH()  -  Sends the character in SR1 to the LCD.  }
out_ch:	dm(temp1) = sr1;		{ Save for second nibble }
	sr = lshift sr1 by -4 (hi);	{ Get high 4 bits }
	ay0 = b#10000000;		{ This says it is data }
	ar = sr1 or ay0;
	call lcd4;
	ar = dm(temp1);			{ Get character again }
	ay0 = b#00001111;
	ar = ar and ay0;
	ay0 = b#10000000;		{ This says it is data }
	ar = ar or ay0;
	jump lcd4;

{ N2BCD()  -  Converts the input in AR to ASCII digits in digit[1] msd to
  digit[5] lsb.  A blank is in digit[0] for positive numbers and a - sign
  for negative numbers.
  Signed (-32768, 32767) input. }
n2bcd:	ar = pass ar;
	ax0 = h#20;			{ Blank for plus }
	if ge jump n2s;
	ax0 = h#2d;			{ - sign for negative }
	ar = -ar;
n2s:	dm(digit) =ax0;
	dm(temp1) = ar;			{ The number to be converted to BCD }
	ay0 = 10000;			{ Find the msdigit }
	ay1 = h#30;			{ Count the subtractions }
n2a:	ar = dm(temp1);			{ Test the current reduced number }
	af = ar - ay0;
	if lt jump n2b;			{ Done for this digit }
	ar = ar - ay0;			{ Not done, reduce working number }
	dm(temp1) = ar;
	ar = ay1 + 1;			{ Increase current digit }
	ay1 = ar;			{ This is where it is kept }
	jump n2a;			{ Continue subtractions }
n2b:	dm(digit + 1) = ay1;

	ay0 = 1000;
	ay1 = h#30;
n2c:	ar = dm(temp1);			{ Test the current reduced number }
	af = ar - ay0;
	if lt jump n2d;			{ Done for this digit }
	ar = ar - ay0;			{ Not done, reduce working number }
	dm(temp1) = ar;
	ar = ay1 + 1;			{ Increase current digit }
	ay1 = ar;			{ This is where it is kept }
	jump n2c;			{ Continue subtractions }
n2d:	dm(digit + 2) = ay1;

	ay0 = 100;			{ Find the msdigit }
	ay1 = h#30;			{ Count the subtractions }
n2e:	ar = dm(temp1);			{ Test the current reduced number }
	af = ar - ay0;
	if lt jump n2f;			{ Done for this digit }
	ar = ar - ay0;			{ Not done, reduce working number }
	dm(temp1) = ar;
	ar = ay1 + 1;			{ Increase current digit }
	ay1 = ar;			{ This is where it is kept }
	jump n2e;			{ Continue subtractions }
n2f:	dm(digit + 3) = ay1;

	ay0 = 10;			{ Find the msdigit }
	ay1 = h#30;			{ Count the subtractions }
n2g:	ar = dm(temp1);			{ Test the current reduced number }
	af = ar - ay0;
	if lt jump n2h;			{ Done for this digit }
	ar = ar - ay0;			{ Not done, reduce working number }
	dm(temp1) = ar;
	ar = ay1 + 1;			{ Increase current digit }
	ay1 = ar;			{ This is where it is kept }
	jump n2g;			{ Continue subtractions }
n2h:	dm(digit + 4) = ay1;

	ay1 = h#30;
	ar = ar + ay1;
	dm(digit + 5) = ar;
	rts;

{ KNOB()  -  Reads opto-encoder knob and updates count in ar.
  The encoder output is a two bit gray count that has the following
  pattern, where movement to the right is CW rotation:
  Input bit 4: 0011001100 etc.
  Input bit 5: 0110011001 etc.
  These two bits are the encoder "state".
  The previous knob state is kept as the two lower bits of dm(knob_st).
  This previous state is shifted left 2 places (x4) and the new state
  is or'ed in to the first two bits.  This 4 bit number determines the
  entry point to a 16 entry table, encoder[], which determines the
  movement of the encoder, -1, 0 or +1 count.  This value is returned
  in ay0.
}
knob:	ay0 = 4;  call inbit;		{ LSB of knob state }
	mr1 = 0;
	ar = tstbit 3 of ax0;
	if eq jump kn1;
	mr1 = 1;
kn1:	ay0 = 5;  call inbit;
	ar = tstbit 3 of ax0;
	if eq jump kn2;
	ar = setbit 1 of mr1;
	mr1 = ar;
kn2:					{ Here with new state in mr1 }
	ar = dm(knob_st);		{ State of knob at last measurement }
	sr = lshift ar by 2 (hi);	{ Move left 2 bits }
	ay0 = sr1;
	ar = mr1 or ay0;		{ 4 bit state }
	dm(knob_st) = mr1;		{ Current state for next time }
	ay0 = ^encoder;
	ar = ar+ay0;
	i4 = ar;  m4 = 0;  l4 = 0;
	ay0 = pm(i4, m4);
	rts;

{ AR2LCD()  -  Puts the value in ar onto the LCD display in the first 6
  characters, blank or sign, and 5 digits.  Enter at ar2lcd1 with
  the LCD address set, to put the number elsewhere on the display.
}
ar2lcd:	dm(temp3) = ar;
	sr1 = h#80;  call out_cmd;	{ 1st character of the  1st line }
	ar = dm(temp3);
ar2lcd1: call n2bcd;			{ Convert ar to bcd }
	i0 = ^digit;
	m0 = 1;  l0 = 0;
	cntr = 6;
	do chlp9 until ce;
	    sr1 = dm(i0, m0);
	    call out_ch;
chlp9:	    nop;
	rts;	

{ SEND_MSG()  -  Sends the contents of msg[] tothe right 8 character
  positions of the LCD.  }
send_msg: sr1 = h#a9;
	call out_cmd;	{ Address right 8 characters }
	i0 = ^msg;
	m0 = 1;  l0 = 0;
	cntr = 8;
	do sm1 until ce;
	    sr1 = dm(i0, m0);
	    call out_ch;
sm1:	    nop;
	rts;

{ INBIT()  -  Gets a hardware input bit.  Bit is selected by ay0 (0,7)
  and the results are returned as ax0 (all PFDATA with the input as bit 3)
  and also as the zero flag in the arithmetic results.  This can be used
  by instructions 'if eq ...' or 'if ne ...'.  }
inbit:	ay1 = b#11111000;		{ First clear 3 bits of data16 }
	ax1 = dm(data16);
	ar = ax1 and ay1;
	ar = ar or ay0;			{ OR in the address of data input }
	dm(data16) = ar;
	call load16;
	ax0 = dm(PFDATA);
	ar = tstbit 3 of ax0;
	rts;

in_sw1:	ay0 = 0;
	jump inbit;

in_sw2:	ay0 = 1;
	jump inbit;

in_sw3:	ay0 = 2;
	jump inbit;

in_sw4:	ay0 = 3;
	jump inbit;

red_on: ar = dm(data16);
	ar = setbit 4 of ar;		{ Turn on red LED }
	dm(data16) = ar;
	jump load16;			{ Send to hardware }

red_off: ar = dm(data16);
	ar = clrbit 4 of ar;		{ Turn off red LED }
	dm(data16) = ar;
	jump load16;			{ Send to hardware }

grn_on: ar = dm(data16);
	ar = setbit 3 of ar;		{ Turn on green LED }
	dm(data16) = ar;
	jump load16;			{ Send to hardware }

grn_off: ar = dm(data16);
	ar = clrbit 3 of ar;		{ Turn off green LED }
	dm(data16) = ar;
	jump load16;			{ Send to hardware }

{ DELAY1()  -  A wait of about 1 microsecond.  }
delay1:	cntr = 29;
	do dy1 until ce;
dy1:	nop;
	rts;

{ DELAY3()  -  A wait of about 3 microsecond.  }
delay3:	cntr = 90;
	do dy3 until ce;
dy3:	nop;
	rts;

{ DELAY50()  -  A wait of 50 microseconds }
delay50: cntr = 1500;
	do dy50 until ce;
dy50:	nop;
	rts;

{ DELAY5M()  -  A wait of 5000 microseconds.  }
delay5m: cntr = 4848;
	do dy5 until ce;
	  call delay1;
dy5:	  nop;				{ Don't put call here }
	rts;

{******************************************************************************
 * Main startup initialization.  Many variables and registers require
 * startup initialization, either for operation or to improve simulation.
 * This routine includes the parts that can easily be simulated
 *****************************************************************************}
init0:	{ i7, l7, i6, l6 devoted to auto-buffering in SPORT0.  Initialized
	  here. These need protection, if used elsewhere in the program.    }
        i7 = ^rx_buf;
        l7 = %rx_buf;
        m7 = 1;
        i6 = ^tx_buf;
        l6 = %tx_buf;
        m6 = 1;
	{ i5, l5, m5 used in next_cmd to load AD1847 regs and then made free. }
        i5 = ^init_cmds;
        l5 = %init_cmds;
        m5 = 1;

        ax0=0xC000; dm(tx_buf)=ax0;    { Mode Change Enable (MCE)  initially }
        ax0=0x0000; dm(tx_buf+1)=ax0;
        ax0=0x0000; dm(tx_buf+2)=ax0;

	{  AD1847  Command Register Initialization.  They are placed into
	   memory here and will be loaded into the AD1847 with interrupt
	   routines in init1.  The details of these registers are given
	   in the AD1847 data sheet.  Note that the destinations are given
	   by the second hex digit and that register addresses b, e and f
	   are not valid, or used.    }
        ax0=0xc002;  dm(init_cmds)=ax0;	   { Left Input control }
        ax0=0xc102;  dm(init_cmds+1)=ax0;  { Right Input control }
        ax0=0xc288;  dm(init_cmds+2)=ax0;  { Left Aux #1 control }
        ax0=0xc388;  dm(init_cmds+3)=ax0;  { Right Aux #1 control }
        ax0=0xc488;  dm(init_cmds+4)=ax0;  { Left Aux #2 control }
        ax0=0xc588;  dm(init_cmds+5)=ax0;  { Right Aux #2 control }
        ax0=0xc680;  dm(init_cmds+6)=ax0;  { Left DAC control }
        ax0=0xc780;  dm(init_cmds+7)=ax0;  { Right DAC control }
        ax0=0xc85c;  dm(init_cmds+8)=ax0;  { Format: 48 KHz, (c850h for 8 KHz)}
        ax0=0xc909;  dm(init_cmds+9)=ax0;  { Interface configuration }
        ax0=0xca00;  dm(init_cmds+10)=ax0; { Pin configuration }
        ax0=0xcc40;  dm(init_cmds+11)=ax0; { Misc Information }
        ax0=0xcd00;  dm(init_cmds+12)=ax0; { Digital Mix control }

{********************* Serial PORT 0 Initialization *************************
 * The following sets up the SPORT0 for autobuffering of 3 receive words
 * (Status, L data, and R data) or 3 transmit words (Command, L data, and
 * R data.)  These are data memory locations, like dm(SPORT0_Autobuf).
 * The actual locations in memory come from the system.k file and the details
 * of the data loaded to these dm locations is given in the Analog Devices
 * "ADSP-2100 Family Users Manual."
 *****************************************************************************}
        ax0= b#0000110101111111;   dm (SPORT0_Autobuf) = ax0;
        ax0 = 0;    dm (SPORT0_RFSDIV) = ax0;
        ax0 = 0;    dm (SPORT0_SCLKDIV) = ax0;
        ax0 = b#1000011000001111;   dm (SPORT0_Control_Reg) = ax0;
        ax0 = b#0000000000000111;   dm (SPORT0_TX_Channels0) = ax0;
        ax0 = b#0000000000000111;   dm (SPORT0_TX_Channels1) = ax0;
        ax0 = b#0000000000000111;   dm (SPORT0_RX_Channels0) = ax0;
        ax0 = b#0000000000000111;   dm (SPORT0_RX_Channels1) = ax0;

{******************* System and memory initialization ***********************
 * A variety of hardware dependent parameters are are set by the following
 * registers.  The first two are in data memory and ifc, icntl and mstst are
 * three non-memory registers that control hardware operation.
 * The actual locations in memory come from the system.k file, whereas the
 * three non-memory registers are understood directly by the assembler.
 * The details of the data loaded to these dm locations is given in the
 * Analog Devices "ADSP-2100 Family Users Manual."
 *****************************************************************************}

	ax0 = 0;			{ No wait states are used }
	dm(DM_Wait_Reg)=ax0;
        ax0 = b#0001000000000000;	{ Enable SPORT0 }
        dm(System_Control_Reg) = ax0;
        ifc = b#0000000011111111;	{ Clear any pending interrupt }
        nop;
	icntl = 0;			{ Interrupts level sensitive, no nest }
	mstat = b#1000000;		{ Primary data registers }
	ax0 = b#0111111100000111;	{ PF0, 1, 2 out, PF3-7 in }
	dm(PFTYPE)=ax0;
	ax0 = 0;
	dm(PFDATA) = ax0;
{>>>}	ax0 = b#1111000011101000;	{ FOR TEST }
	dm(data16)=ax0;			{ 16 bits of shift data }
	rts;

{*****************************************************************************
 * Start up initialization of the AD1847 Codec.  This requires some effort to
 * simulate, and it is easiest to skip this part when running the sim21
 * simulator.  This method doesn't support all testing, but is OK for most
 * purposes.
 * This part of the program is the "background" routine during the
 * initialization of the AD1847 Codec.  The SPORT0 Tx interrupt is enabled.
 * The transmission of data to the Codec will program the
 * Codec control registers.  See the AD1847 data sheet for the myriads of
 * details involved.
 *****************************************************************************}
init1:	{ AD 1847 Codec Initialization
	  The stat_flag=0 will prevent reinit of AD1847 from SPORT0 TX ints }
	ax0 = 1;			{ Set flag so control words will be }
	dm(stat_flag) = ax0;		{   be loaded by next_cmd routine }
	imask = b#0001000000;		{ Enable SPORT0 Tx int, disable others}
	ax0 = dm(i6, m6);		{ Start interrupt }
	tx0 = ax0;
check_init:
        ax0 = dm (stat_flag);		{ Wait for entire initialization }
        af = pass ax0;			{   command buffer (init_cmds) to be }
        if ne jump check_init;		{   sent to the codec by next_cmd   }
        ay0 = 2;
check_acih:
        ax0 = dm (rx_buf);		{ Once initialized, wait for codec }
        ar = ax0 and ay0;		{   to come out of autocalibration }
        if eq jump check_acih;		{ Wait for bit set }
check_aci1:
        ax0 = dm (rx_buf);		{ Wait for bit clear }
        ar = ax0 and ay0;
        if ne jump check_aci1;
	idle;				{ Sit and wait for an interrupt }

        ay0 = 0xbf3f;			{ Unmute left DAC }
        ax0 = dm (init_cmds + 6);
        ar = ax0 AND ay0;
        dm (tx_buf) = ar;
        idle;

        ax0 = dm (init_cmds + 7);	{ Unmute right DAC }
        ar = ax0 AND ay0;
        dm (tx_buf) = ar;
        idle;

	ax0 = 0xc901;			{ Clear autocalibration request }
	dm(tx_buf)=ax0;
	idle;

        ax0 = 0x8000;			{ Clear over-range flags }
        dm (tx_buf) = ax0;
        idle;

        ifc = b#00000011111111;		{ Clear any pending interrupt }
        nop;
        imask = b#0000100000;		{ Enable RX0 interrupt, now our only }
	rts;


{************************  NEXT_CMD  ****************************************
 * Transmit Interrupt Service Routine used for Codec initialization
 * DAG 5 registers used here, but are available after this routine is run.
 * The commands are loaded here into the command location dm(tx_buf+0) where
 * they will be transferred to the Codec.
 * Adapted from EZ-KIT Lite routines.
 ****************************************************************************}
next_cmd:
        ena sec_reg;			{ Get other set of registers }
        ax0 = dm (i5, m5);		{ Get next control word and }
        dm (tx_buf+0) = ax0;		{   place in command word }
        ax0 = i5;			{ Now see if finished by comparing }
        ay0 = ^init_cmds;		{   the i5 address with the starting}
        ar = ax0 - ay0;			{   i5 value. Circ buff brings here}
        if gt rti;                  	{ Exit if more control words waiting }
        ax0 = 0x8000;			{ else clear the Mode Change Enable }
        dm (tx_buf+0) = ax0;		{   (MCE) bit in the Codec cntrl word}
        ax0 = 0;			{ Flag the completion by clearing }
        dm (stat_flag) = ax0;		{   stat_flag, stopping calls to here }
	dis sec_reg;
        rti;

{************************  INIT_LCD  ****************************************
 * The following is a prescribed initialization routine for 4-bit
 * operation of the Optrex DMC16117A LCD.
 ****************************************************************************}
init_lcd:
	ar = h#03;
	call lcd4;
	call delay5m;

	ar = h#03;
	call lcd4;
	call delay50;
	call delay50;

	ar = h#03;
	call lcd4;

	ar = h#02;			{ Fcn set, 4 bits }
	call lcd4;

	sr1 = h#28;  call out_cmd;	{ Fcn set, 2 lines, 5x8 }
	sr1 = h#01;  call out_cmd;	{ Clear screen }
	sr1 = h#06;  call out_cmd;	{ Increment, no shift }
	sr1 = h#0C;  jump out_cmd;	{ Display on, no cursor }
	{ End of LCD initialization }

.endmod;
